package org.gradle.starter;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FilenameFilter;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.net.URL;
import java.net.URLClassLoader;

/**
 * The Class GradleStarter - used as a main entry point for gradle.exe
 * Based on JAVA_HOME and GRADLE_HOME it dynamically loads gradle jar and tools.jar
 * into the classpath, and sets JVM settings gradle.home and tools.jar accordingly.
 *
 * @author Adam Pohorecki <adam@pohorecki.pl>
 */
public class GradleStarter {

    /** Default JAVA_HOME - used as a last resort */
    private static final File DEFAULT_JAVA_HOME = new File(
	    "C:\\Program Files\\jre1.6.0_04");

    /**
     * The entry point
     *
     * @param args the command line arguments
     *
     * @throws Exception the exceptions are propagated
     */
    public static void main(final String[] args) throws Exception {
	new GradleStarter().run(args);
    }

    /**
     * Run Gradle with given command line arguments.
     *
     * @param args the command line arguments as in the main method
     *
     * @throws Exception the exception
     */
    @SuppressWarnings("unchecked")
    protected void run(final String[] args) throws Exception {
	final String gradleHome = getGradleHome();
	final String javaHome = getJavaHome();

	System.out.println("Using GRADLE_HOME="+gradleHome);
	System.out.println("Using JAVA_HOME="+javaHome);

	final File gradleJar = getGradleJar(gradleHome);
	final File toolsJar = getToolsJar(javaHome);

	final URL[] urls = new URL[] { gradleJar.toURL(), toolsJar.toURL() };
	final URLClassLoader classLoader = loadAdditionalJars(urls);
	final Class toolsMain = classLoader.loadClass("org.gradle.ToolsMain");
	final Method mainMethod = toolsMain.getMethod("main", String[].class);

	System.setProperty("gradle.home", gradleHome);
	System.setProperty("tools.jar", toolsJar.getAbsolutePath());

	mainMethod.invoke(null, new Object[] {args});
    }

    /**
     * Load additional jars into the classpath.
     *
     * @param urls the urls of the jars
     *
     * @return the class loader, that is able to load provided jars
     *
     * @throws NoSuchMethodException there is no method addURL in URLClassLoader
     * @throws IllegalAccessException
     * @throws InvocationTargetException when something goes wrong with addURL invocation
     */
    protected URLClassLoader loadAdditionalJars(final URL[] urls)
	    throws NoSuchMethodException, IllegalAccessException,
	    InvocationTargetException {
	URLClassLoader classLoader = null;
	if (ClassLoader.getSystemClassLoader() instanceof URLClassLoader) {
	    classLoader = (URLClassLoader) ClassLoader.getSystemClassLoader();

	    final Class<URLClassLoader> urlClassLoader = URLClassLoader.class;
	    final Method addURL = urlClassLoader.getDeclaredMethod("addURL", new Class[] {URL.class});
	    addURL.setAccessible(true);

	    for (final URL url : urls) {
		addURL.invoke(classLoader, new Object[] {url});
	    }
	} else {
	    classLoader = new URLClassLoader(urls);
	    Thread.currentThread().setContextClassLoader(classLoader);
	}
	return classLoader;
    }



    /**
     * Gets the tools jar file.
     *
     * @param javaHome the java home path
     *
     * @return the tools jar
     *
     * @throws FileNotFoundException when the file tools.jar was not found
     */
    protected File getToolsJar(final String javaHome)
	    throws FileNotFoundException {
	final File jarFile = new File(javaHome + File.separator + "lib"
		+ File.separator + "tools.jar");
	if (!jarFile.exists()) {
	    throw new FileNotFoundException(
		    "tools.jar can not be found. Expected to find it in: "
			    + jarFile.getAbsolutePath());
	}
	return jarFile;
    }

    /**
     * Gets the Java home path.
     *
     * @return the Java home path
     */
    protected String getJavaHome() {
	String javaHome = System.getenv().get("JAVA_HOME");
	if (javaHome == null) {
	    javaHome = DEFAULT_JAVA_HOME.getAbsolutePath();
	}
	return javaHome;
    }

    /**
     * Gets the Gradle jar file.
     *
     * @param gradleHome the Gradle home path
     *
     * @return the Gradle jar file
     *
     * @throws FileNotFoundException when the Gradle jar file is not found
     */
    protected File getGradleJar(final String gradleHome)
	    throws FileNotFoundException {
	final File dir = new File(gradleHome + File.separator + "lib");
	final FilenameFilter filter = new FilenameFilter() {
	    public boolean accept(File dir, String name) {
		return name.matches("^gradle-\\d.*\\.jar$");
	    }
	};
	final File[] matching = dir.listFiles(filter);
	if (matching.length == 0) {
	    System.out.println("GRADLE_HOME(\"" + gradleHome
		    + "\") is not a valid gradle home directory!");
	    throw new FileNotFoundException("Gradle jar file was not found in "
		    + dir);
	} else if (matching.length > 1) {
	    System.out.println("GRADLE_HOME(\"" + gradleHome
		    + "\") contains more than one matching file!" + matching);
	}
	return matching[0];
    }

    /**
     * Gets the Gradle home path.
     *
     * @return the Gradle home path
     */
    protected String getGradleHome() {
	final String gradleHome = System.getenv().get("GRADLE_HOME");

	if (gradleHome != null) {
	    final File dir = new File(gradleHome);
	    if (dir.exists()) {
		if (dir.isDirectory()) {
		    return dir.getAbsolutePath();
		} else {
		    System.out.println("GRADLE_HOME(\"" + gradleHome
			    + "\") exists, but it's not a directory!");
		}
	    } else {
		System.out.println("GRADLE_HOME(\"" + gradleHome
			+ "\") does not exist!");
	    }
	} else {
	    System.out.println("GRADLE_HOME is not set!");
	}

	return new File(System.getProperty("user.dir")).getAbsolutePath();
    }
}
